"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const subscribe_1 = require("./subscribe");
const node_timers_1 = require("node:timers");
const sinon = __importStar(require("sinon"));
const vitest_1 = require("vitest");
const defaultTrigger = {
    expression: ['match', 'foo', 'basename'],
    id: 'foo',
    initialRun: true,
    interruptible: false,
    name: 'foo',
    onChange: async () => { },
    onTeardown: async () => { },
    relativePath: 'foo',
    retry: {
        retries: 0,
    },
    throttleOutput: {
        delay: 0,
    },
    watch: 'foo',
};
const wait = (time) => {
    return new Promise((resolve) => {
        (0, node_timers_1.setTimeout)(resolve, time);
    });
};
(0, vitest_1.it)('evaluates onChange', async () => {
    const abortController = new AbortController();
    const trigger = {
        ...defaultTrigger,
        abortSignal: abortController.signal,
    };
    const subscriptionMock = sinon.mock(trigger);
    const onChange = subscriptionMock
        .expects('onChange')
        .once()
        .callsFake(() => {
        abortController.abort();
        return Promise.resolve(null);
    });
    const subscription = (0, subscribe_1.subscribe)(trigger);
    subscription.trigger([]);
    (0, vitest_1.expect)(subscriptionMock.verify());
    (0, vitest_1.expect)(onChange.args[0][0].taskId).toMatch(/^[a-z\d]{8}$/u);
});
(0, vitest_1.it)('waits for onChange to complete when { interruptible: false }', async () => {
    const abortController = new AbortController();
    const trigger = {
        ...defaultTrigger,
        abortSignal: abortController.signal,
        interruptible: false,
    };
    const triggerMock = sinon.mock(trigger);
    const onChange = triggerMock.expects('onChange').twice();
    let completed = false;
    onChange.onFirstCall().callsFake(async () => {
        await wait(100);
        completed = true;
    });
    onChange.onSecondCall().callsFake(() => {
        (0, vitest_1.expect)(completed).toBe(true);
        abortController.abort();
    });
    const subscription = (0, subscribe_1.subscribe)(trigger);
    await subscription.trigger([]);
    await subscription.trigger([]);
    (0, vitest_1.expect)(onChange.callCount).toBe(2);
});
(0, vitest_1.it)('waits for onChange to complete when { interruptible: true } when it receives a shutdown signal', async () => {
    const abortController = new AbortController();
    const trigger = {
        ...defaultTrigger,
        abortSignal: abortController.signal,
    };
    let resolved = false;
    const subscriptionMock = sinon.mock(trigger);
    subscriptionMock
        .expects('onChange')
        .once()
        .callsFake(() => {
        return new Promise((resolve) => {
            (0, node_timers_1.setTimeout)(() => {
                resolved = true;
                resolve(null);
            }, 100);
        });
    });
    const subscription = (0, subscribe_1.subscribe)(trigger);
    setImmediate(() => {
        abortController.abort();
    });
    await subscription.trigger([]);
    (0, vitest_1.expect)(subscriptionMock.verify());
    (0, vitest_1.expect)(resolved).toBe(true);
});
(0, vitest_1.it)('throws if onChange produces an error', async () => {
    const abortController = new AbortController();
    const trigger = {
        ...defaultTrigger,
        abortSignal: abortController.signal,
    };
    const subscriptionMock = sinon.mock(trigger);
    subscriptionMock.expects('onChange').rejects(new Error('foo'));
    const subscription = (0, subscribe_1.subscribe)(trigger);
    await (0, vitest_1.expect)(subscription.trigger([])).rejects.toThrowError('foo');
    await abortController.abort();
});
(0, vitest_1.it)('retries failing routines', async () => {
    const abortController = new AbortController();
    const trigger = {
        ...defaultTrigger,
        abortSignal: abortController.signal,
        retry: {
            retries: 1,
        },
    };
    const subscriptionMock = sinon.mock(trigger);
    const onChange = subscriptionMock.expects('onChange').twice();
    onChange.onFirstCall().rejects(new Error('foo'));
    onChange.onSecondCall().callsFake(() => {
        abortController.abort();
        return Promise.resolve(null);
    });
    const subscription = await (0, subscribe_1.subscribe)(trigger);
    await subscription.trigger([]);
    (0, vitest_1.expect)(onChange.verify());
});
(0, vitest_1.it)('reports { first: true } only for the first event', async () => {
    const abortController = new AbortController();
    const trigger = {
        ...defaultTrigger,
        abortSignal: abortController.signal,
    };
    const subscriptionMock = sinon.mock(trigger);
    const onChange = subscriptionMock.expects('onChange').twice();
    onChange.onFirstCall().resolves(null);
    onChange.onSecondCall().callsFake(() => {
        abortController.abort();
        return Promise.resolve(null);
    });
    const subscription = (0, subscribe_1.subscribe)(trigger);
    await subscription.trigger([]);
    await subscription.trigger([]);
    (0, vitest_1.expect)(onChange.args).toMatchObject([
        [
            {
                first: true,
            },
        ],
        [
            {
                first: false,
            },
        ],
    ]);
    (0, vitest_1.expect)(subscriptionMock.verify());
});
//# sourceMappingURL=subscribe.test.js.map