"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.testExpression = void 0;
const micromatch_1 = __importDefault(require("micromatch"));
const node_path_1 = __importDefault(require("node:path"));
const testExpression = (expression, fileName) => {
    const name = expression[0];
    if (name === 'allof') {
        const nextExpressions = expression.slice(1);
        return nextExpressions.every((nextExpression) => {
            return (0, exports.testExpression)(nextExpression, fileName);
        });
    }
    if (name === 'anyof') {
        const nextExpressions = expression.slice(1);
        return nextExpressions.some((nextExpression) => {
            return (0, exports.testExpression)(nextExpression, fileName);
        });
    }
    if (name === 'dirname') {
        const patternInput = expression[1];
        if (patternInput.startsWith('/')) {
            throw new Error('dirname cannot start with /');
        }
        if (patternInput.endsWith('/')) {
            throw new Error('dirname cannot end with /');
        }
        const pattern = '/' + patternInput;
        const lastIndex = node_path_1.default.dirname(fileName).lastIndexOf(pattern);
        return lastIndex !== -1;
    }
    if (name === 'idirname') {
        const patternInput = expression[1];
        if (patternInput.startsWith('/')) {
            throw new Error('dirname cannot start with /');
        }
        if (patternInput.endsWith('/')) {
            throw new Error('dirname cannot end with /');
        }
        const pattern = '/' + patternInput.toLowerCase();
        const lastIndex = node_path_1.default.dirname(fileName.toLowerCase()).lastIndexOf(pattern);
        return lastIndex !== -1;
    }
    if (name === 'match') {
        const pattern = expression[1];
        const subject = expression[2] === 'basename' ? node_path_1.default.basename(fileName) : fileName;
        return micromatch_1.default.isMatch(subject, pattern, {
            dot: true,
        });
    }
    if (name === 'imatch') {
        const pattern = expression[1];
        const subject = expression[2] === 'basename' ? node_path_1.default.basename(fileName) : fileName;
        return micromatch_1.default.isMatch(subject.toLowerCase(), pattern.toLowerCase(), {
            dot: true,
        });
    }
    if (name === 'not') {
        const subExpression = expression[1];
        return !(0, exports.testExpression)(subExpression, fileName);
    }
    throw new Error('Unknown expression');
};
exports.testExpression = testExpression;
//# sourceMappingURL=testExpression.js.map